import pytest
from django.contrib.auth import get_user_model
from faker import Faker

from pirates.auth import PiratesOIDCAuthenticationBackend

pytestmark = pytest.mark.django_db

fake = Faker()


@pytest.fixture
def backend():
    return PiratesOIDCAuthenticationBackend()


def test_auth_backend__get_sso_id(backend):
    sub = fake.random_letters()
    claims = {"sub": sub}
    assert backend.get_sso_id(claims) == sub


def test_auth_backend__filter_users_by_claims__missing_sub_in_claims(backend):
    claims = {}
    users = backend.filter_users_by_claims(claims)
    assert users.exists() is False


def test_auth_backend__filter_users_by_claims__unknown_user(backend):
    claims = {"sub": fake.random_letters()}
    users = backend.filter_users_by_claims(claims)
    assert users.exists() is False


def test_auth_backend__filter_users_by_claims__known_user(backend, user):
    claims = {"sub": user.sso_id}
    users = backend.filter_users_by_claims(claims)
    assert list(users) == [user]


def test_auth_backend__create_user(backend):
    claims = {
        "sub": fake.random_letters(),
        "given_name": fake.first_name(),
        "family_name": fake.last_name(),
        "email": fake.email(),
    }
    user = backend.create_user(claims)
    assert user.sso_id == claims["sub"]
    assert user.first_name == claims["given_name"]
    assert user.last_name == claims["family_name"]
    assert user.email == claims["email"]
    assert get_user_model().objects.get() == user


def test_auth_backend__update_user(backend, user):
    claims = {
        "sub": user.sso_id,
        "given_name": fake.first_name(),
        "family_name": fake.last_name(),
        "email": fake.email(),
    }
    assert user.first_name != claims["given_name"]
    assert user.last_name != claims["family_name"]
    assert user.email != claims["email"]
    updated_user = backend.update_user(user, claims)
    assert updated_user.sso_id == user.sso_id
    assert updated_user.first_name == claims["given_name"]
    assert updated_user.last_name == claims["family_name"]
    assert updated_user.email == claims["email"]
    assert get_user_model().objects.get() == updated_user
